
import pylab
import os

from variables import *

log10_for_variance_treshhold = 100
colors = ['b', 'g', 'r', 'c', 'm', 'y', 'burlywood', 'chartreuse', 'k']

class Run (object):
    def __init__(self, name=None):
        if name:
            self.outputf = 'output_'+name+'.dat'
            self.varf = 'var_output_'+name+'.dat'
            self.timesf = 'times_'+name+'.py'
            self.configf = 'config_'+name+'.py'
            self.runtimef = 'runtime_'+name+'.py'
            self.movief = 'movie_'+name
            self.name = name
        else:
            self.outputf = 'output.dat'
            self.varf = 'var_output.dat'
            self.timesf = 'times.py'
            self.configf = 'config.py'
            self.runtimef = 'runtime.py'
            self.movief = 'movie'
            self.name = 'run'
        times = {}
        execfile(self.timesf, globals(), times)
        self.times = times['time']
        config = {}
        execfile(self.configf, globals(), config)
        self.opts = config['opts']
        self.vars = config['used']
        self.runtime = {}
        execfile(self.runtimef, globals(), self.runtime)
        self.i = Config(self, int)
        self.f = Config(self, float)
        self.nx = self.i.NX
        self.ny = self.i.NY
        self.nz = self.i.NZ
        
        # extracting the data (infolines) to be displayed on the plots
        self.info_opts = self.opts
        self.info_vars = config['vars']
        if 'RUNTIME_FILE' in self.info_vars: del self.info_vars ['RUNTIME_FILE']
        if 'LOG_FILE'     in self.info_vars: del self.info_vars ['LOG_FILE']
        if 'TIME_FILE'    in self.info_vars: del self.info_vars ['TIME_FILE']
        if 'DATA_FILE'    in self.info_vars: del self.info_vars ['DATA_FILE']
        
        # detecting number of variables from equation type
        if 'equation' in self.opts:
          self.NVARS = get_NVARS (self.opts['equation'])
        else:
          self.NVARS = 16
    
    def var(self, v, ts, case="mean"):
        if ts > len(self.times):
            raise Exception, "Timestep out of bounds"
        if case == "mean":
            f = open(self.outputf, 'rb')
        else: # case="variance"
            f = open(self.varf, 'rb')
        n = self.nx*self.ny*self.nz
        f.seek(8*n*(v.i + self.NVARS*ts))
        fd = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        fd.shape = (self.nx, self.ny, self.nz)
        return fd
    
    def showInfo(self, prefix='', adjust=True):
      
      string_opts = ''.join([' %s:%s,' % x for x in sorted(self.info_opts.items())])
      string_vars = ''.join([' %s=%s' % x for x in sorted(self.info_vars.items())])
      
      text  = prefix + 'OPTS:' + string_opts[0:-1] + '\n'
      text += prefix + 'VARS:' + string_vars + '\n'
      
      if 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime']
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      text += prefix + 'INFO: ' + 'nodes:  ' + str(self.runtime['nodes']) + ', ' + 'runtime: ' + runtime
      
      if 'mpi_scaling' in self.runtime:
        text += ', ' + 'efficiency: ' + str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'%'
      
      pylab.figtext(0.01, 0.015, text, fontsize='small')
      
      if adjust:
        pylab.subplots_adjust(bottom=0.17)
    
    def plotTopography_1d(self, plt, xr, ts):
        b_mean = self.var(sw.B, ts, "mean")[:,0,0]
        b_var = self.var(sw.B, ts, "variance")[:,0,0]
        b_std = numpy.sqrt(b_var)
        
        plt.plot(xr, b_mean, "k-", label="topography")
        ymin, ymax = pylab.ylim()
        plt.fill_between(xr, b_mean, ymin, alpha=0.5, color='black')
        plt.plot(xr, b_mean + b_std, "k--")#, label="bed std")
    
    def plotTopography_2d(self, plt, extent, ts, case, N=3):
        b = self.var(sw.B, ts, case)[:,:,0]
        cp = plt.contour(b, N, extent=extent, cmap=pylab.copper(), alpha=0.5)
        fmt = '%2.1f'
        if case == "variance": fmt = '%1.1e'
        plt.clabel(cp, fmt=fmt, inline=1, fontsize=8)
    
    def plot(self, v, frame=False, ts=None, extent=None, infolines=True, save=None, topography=True, loc='lower left', cut_slice=None):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.plot_1d(v, ts, extent, frame, infolines, save, topography, loc)
        else:
            self.subplot_2d(v, ts, extent, extent, frame, infolines, save, topography, cut_slice)
    
    def plot0(self, v, frame=False, ts=None, extent=None, infolines=True, save=None, topography=True, loc='lower left', cut_slice=None):
        if not int(self.vars['SAVE0']):
            raise Exception, "Snapshot at t=0 is not available! Use SAVE0=1 when compiling."
        
        if ts is None: ts = len(self.times)-1
        
        if self.ny == 1:
            if not frame:
              pylab.figure(figsize=(2*8,6))
            pylab.subplot(121)
            self.plot_1d(v, 0, extent, True, False, topography=topography, loc=loc)
            pylab.subplot(122)
            self.plot_1d(v, ts, extent, frame, infolines, save, topography=topography, loc=loc)
        else:
            self.subplot_2d(v, 0, extent,  extent, frame, infolines, save, topography, cut_slice)
            self.subplot_2d(v, ts, extent, extent, frame, infolines, save, topography, cut_slice)
    
    def subplot(self, v, frame=False, ts=None, extent_mean=None, extent_var=None, infolines=True, save=None, topography=True, movie=False, loc='lower left', cut_slice=None):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.subplot_1d(v, ts, extent_mean, extent_var, frame, infolines, save, movie, loc=loc, topography=topography)
        else:
            self.subplot_2d(v, ts, extent_mean, extent_var, frame, infolines, save, topography, cut_slice=cut_slice, movie=movie)

    def plot_1d(self, v, ts, extent=None, frame=False, infolines=True, save=None, x=None, y=None, z=None, topography=True, loc='lower left'):
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
        mean = self.var(v, ts, "mean")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          mean += self.var(sw.B, ts, "mean")
        
        pylab.plot(xr, mean[:,0,0], label="mean")
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          var = self.var(v, ts, "variance")
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            var -= self.var(sw.B, ts, "variance")
          std = numpy.sqrt(numpy.fabs(var));
          upper_std = mean + std;
          lower_std = mean - std;
          pylab.plot(xr, upper_std[:,0,0], "--", label="mean + std")
          pylab.plot(xr, lower_std[:,0,0], "--", label="mean - std")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and v == sw.H:
          self.plotTopography_1d(pylab.gca(), xr, ts)
        
        if extent:
            pylab.ylim(*extent)
        pylab.title(v.name + ' at t=%.2f' % self.times[ts])
        pylab.legend(loc=loc)
        
        pylab.subplots_adjust(right=0.96)
        pylab.subplots_adjust(left=0.06)
        pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if not frame:
            pylab.show()
        if save:
            pylab.savefig(save)
    
    def subplot_1d(self, v, ts, extent_mean=None, extent_var=None, frame=False, infolines=True, save=None, movie=False, x=None, y=None, z=None, topography=True, loc='lower left'):
        
        if movie:
          pylab.figure(figsize=(2*8,6))
        
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        pylab.plot(xr, d[:,0,0], label=self.name)
        pylab.title('mean' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
        pylab.legend(loc=loc)
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and v == sw.H:
          self.plotTopography_1d(pylab.gca(), xr, ts)
        
        if extent_mean:
            pylab.ylim(*extent_mean)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          
          pylab.subplot(122)
          pylab.plot(xr, d[:,0,0], label=self.name)
          if extent_var:
              pylab.ylim(*extent_var)
          if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          else:
            pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f' % self.times[ts])
          pylab.legend(loc=loc)
        
        pylab.subplots_adjust(right=0.98)
        pylab.subplots_adjust(left=0.03)
        pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if not frame and not movie:
            pylab.show()
        if save:
            pylab.savefig(save)
    
    def subplot_2d(self, v, ts, extent_mean=None, extent_var=None, frame=False, infolines=True, save=None, topography=True, cut_slice=None, movie=False):
        
        if cut_slice:
          self.subplot_2d_slice (v, ts, extent_mean, extent_var, frame, infolines, save, cut_slice, movie)
          return
        
        if not frame:
          pylab.figure(figsize=(13,6))
        
        # mean
        s = {}
        if extent_mean:
          s['vmin'] = extent_mean[0]
          s['vmax'] = extent_mean[1]
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        
        pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
        pylab.title('mean' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography:
          self.plotTopography_2d(pylab.gca(), (0,self.f.MAXX,0,self.f.MAXY), ts, "mean")
        
        # MUST be before all 'pylab.colorbar()' or otherwise plot is distorted (any idea why?)
        if infolines: self.showInfo()
        pylab.subplots_adjust(right=1.00)
        pylab.subplots_adjust(left=0.00)
        pylab.subplots_adjust(top=0.94)
        pylab.subplots_adjust(wspace=0.05)
        
        pylab.colorbar()
        if 'equation' in self.opts and self.opts['equation'] == 'sw':
          pylab.winter()
        
        # variance
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          s = {}
          if extent_var:
            s['vmin'] = extent_var[0]
            s['vmax'] = extent_var[1]
          
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          
          pylab.subplot(122)
          
          if extent_var:
            pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          elif numpy.max(d[:,:,0]) >= log10_for_variance_treshhold:
            pylab.imshow(numpy.log10(numpy.transpose(d[:,:,0])), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('log10 of ' + 'variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          else:
            pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography:
            self.plotTopography_2d(pylab.gca(), (0,self.f.MAXX,0,self.f.MAXY), ts, "variance")
          
          pylab.colorbar()
          if 'equation' in self.opts and self.opts['equation'] == 'sw':
            pylab.winter()
        
        if not frame and not movie:
          pylab.show()
        if save:
          pylab.savefig(save)
    
    def subplot_2d_slice(self, v, ts, extent_mean, extent_var, frame, infolines, save, cut_slice, movie):
        
        if not frame:
          pylab.figure(figsize=(13,6))
        
        xr = []
        slice_at = 0
        if cut_slice['dir'] == 'X':
          xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
          slice_at = round(float(cut_slice['pos']) / self.f.MAXX * self.nx)
        else:
          xr = numpy.arange(self.ny)*(self.f.MAXY/self.ny)
          slice_at = round(float(cut_slice['pos']) / self.f.MAXY * self.ny)
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        if cut_slice['dir'] == 'X':
          pylab.plot(xr, d[slice_at,:,0], label=self.name)
          pylab.title('mean' + ' of ' + v.name + ' at t=%.2f | sliced at x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
        else:
          pylab.plot(xr, d[:,slice_at,0], label=self.name)
          pylab.title('mean' + ' of ' + v.name + ' at t=%.2f | sliced at y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
        pylab.legend(loc='upper right')
        
        if extent_mean:
            pylab.ylim(*extent_mean)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          d = self.var(v, ts, 'variance')
          pylab.subplot(122)
          if cut_slice['dir'] == 'X':
            pylab.plot(xr, d[slice_at,:,0], label=self.name)
            if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
              pylab.title('variance' + ' of ' + v.name + ' at t=%.2f | sliced at x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
            else:
              pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f | sliced at x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
          else:
            pylab.plot(xr, d[:,slice_at,0], label=self.name)
            if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
              pylab.title('variance' + ' of ' + v.name + ' at t=%.2f | sliced at x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
            else:
              pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f | sliced at x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
          if extent_var:
              pylab.ylim(*extent_var)
          pylab.legend(loc='upper right')
        
        pylab.subplots_adjust(right=0.98)
        pylab.subplots_adjust(left=0.03)
        pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if not frame and not movie:
            pylab.show()
        if save:
            pylab.savefig(save)
    
    def getExtents (self, v, ts, margin=0.0):
      mean = self.var(v, ts, "mean")
      if 'equation' in self.opts and self.opts['equation'] == 'sw':
        mean += self.var(sw.B, ts, "mean")
      
      low  = numpy.min(mean)
      high = numpy.max(mean)
      diff = high - low
      
      low  -= diff * margin
      high += diff * margin
      
      ext_mean = (low, high)
      
      if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
        var  = self.var(v, ts, "variance")
        low  = numpy.min(var)
        high = numpy.max(var)
        diff = high - low
        
        low  -= diff * margin
        high += diff * margin
      
      ext_var = (low, high)
      
      return [ext_mean, ext_var]
    
    def movie(self, v, slowdown=5, infolines=True, last_extent=False, cut_slice=None, topography=True):
        
        margin = 0.1
        
        # for 2d plots, colorbars also indicate both extrema, hence should not be modified
        if self.i.NY != 1:
          margin = 0.0
        
        if last_extent:
          [ext_mean, ext_var] = self.getExtents(v, len(self.times)-1, margin)
        else:
          [ext_mean, ext_var] = self.getExtents(v, 0, margin)
          
          for ts in range(1,len(self.times)):
              
              [ext_mean_tmp, ext_var_tmp] = self.getExtents(v, ts, margin)
              
              ext_mean = (min(ext_mean_tmp[0], ext_mean[0]), max(ext_mean_tmp[1], ext_mean[1]))
              ext_var  = (min(ext_var_tmp [0], ext_var [0]), max(ext_var_tmp [1], ext_var [1]))
        
        for ts in range(len(self.times)):
            self.subplot(v, False, ts, ext_mean, ext_var, infolines, topography=topography, movie=True, cut_slice=cut_slice)
            pylab.savefig('movie_%04d.png' % ts)
            pylab.close()
        
        fps = round ( int(self.vars['NSAVES']) / float(self.vars['MAXT']) )
        
        os.system('mencoder "mf://*.png" -mf fps=%d -o %s_%s.mp4 -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=800' % (fps/slowdown, self.movief, v.name))
        #os.system('mencoder "mf://*.png" -mf fps=%d -o %s.avi -ovc lavc -lavcopts vcodec=msmpeg4v2:vbitrate=800' % (fps/slowdown, self.movief))
        
        os.system('rm movie_*.png')
    
    def matlab(self, v, ts=None, path=''):
        if ts is None: ts = len(self.times)-1
        data = {}
        
        for case in ['mean', 'variance']:
          data[case] = self.var(v, ts, case)[:,0,0]
        
        if 'wall_runtime' in self.runtime:
          data['time'] = self.runtime['wall_runtime']
        else:
          data['time'] = self.runtime['clock_runtime']
        
        data['x'] = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
        
        from scipy.io import savemat
        savemat(path + self.name + '_' + v.name + '.mat', data)
    
    def bfield(self, ts=None):
        if ts is None: ts = len(self.times)-1
        vx,vy = (BX, BY)
        fx = self.var(vx, ts)[:,:,0]
        fy = self.var(vy, ts)[:,:,0]
        fieldlines(fx, fy, self.f.MAXX/self.f.NX, self.f.MAXY/self.f.NY)
    
    def save(self, v, ts=None):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.save_1d(v, ts)
        else:
            self.save_2d(v, ts)
    
    def save_1d(self, v, ts):
        x = numpy.arange(self.nx)*(self.f.MAXX/self.nx)
        d = self.var(v, ts)
        pylab.plot(x, d[:,0,0], label=self.name)
        pylab.title(v.name+' t='+str(self.times[ts]))
        pylab.legend()
        pylab.savefig(self.name+'.png')
    
    def save_2d(self, v, ts):
        d = self.var(v, ts)
        pylab.figure(figsize=(8,6))
        pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower',
                     extent=(0,self.f.MAXX,0,self.f.MAXY))
        pylab.colorbar()
        pylab.title(v.name+' t='+str(self.times[ts]))
        pylab.savefig(self.name+'.png')
    
    def DivB2d(self, per, ts=None,):
        fmt="%10.1e"
        if ts is None: ts = len(self.times)-1
        f = open(self.outputf, 'rb')
        n = self.nx*self.ny
        f.seek(8*n*(BX.i + self.NVARS*ts))
        bx = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        bx.shape = (self.nx, self.ny)
        f = open(self.outputf, 'rb')
        n = self.nx*self.ny
        f.seek(8*n*(BY.i + self.NVARS*ts))
        by = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        by.shape = (self.nx, self.ny)
        if per == 1:
            bxe = numpy.vstack((bx[-1,:],bx,bx[0,:]))
        else:
            bxe = numpy.vstack((bx[0,:],bx,bx[-1,:]))
        by = numpy.transpose(by)
        if per==1:
            bye = numpy.transpose(numpy.vstack((by[-1,:],by,by[0,:])))
        else:
            bye = numpy.transpose(numpy.vstack((by[0,:],by,by[-1,:])))
        div = (bxe[2:self.nx+2,:] - bxe[0:self.nx,:])/(2*self.f.MAXX/self.nx) + (bye[:,2:self.ny+2] - bye[:,0:self.ny])/(2*self.f.MAXY/self.ny)
        divL1 = abs(div).sum()/(self.f.MAXX*self.f.MAXY*reduce((lambda x,y:x*y), div.shape))
        print (r"divL1= %s"% (fmt)) % divL1
    
    def plotLinf(self, v, refd=None, ts=None):
        if ts is None: ts = len(self.times)-1
        d = self.var(v, ts)
        if refd is None:
            max = numpy.max(abs(d))
        else:
            max = numpy.max(abs(d - refd))/numpy.max(abs(refd))
        print (r"Linf= %s"% max)
    
    def plotmin(self, v, ts=None):
        fmt="%10.2e"
        if ts is None: ts = len(self.times)-1
        d = self.var(v, ts)
        min = numpy.min(abs(d))
        print (r"min= %s \\"% (fmt)) % min

def L1(data, refd=None):
    if refd is None:
        n = self.f.MAXX*self.f.MAXY*reduce((lambda x,y:x*y), data.shape)
        return abs(data).sum()/n
    else:
        return abs(data - refd).sum()/abs(refd).sum()

def Linf(data, refd=None):
    if refd is None:
        return numpy.max(abs(data))
    else:
        return numpy.max(abs(data - refd))/numpy.max(abs(refd))

def tableref(r, runref, v):
    fmt="%10.1e"
    scale=1
    t1 = len(r.times)-1
    t2 = len(runref.times)-1
    d = r.var(v, t1)
    refd = runref.var(v, t2)
    zoom = runref.i.NX/r.i.NX
    if runref.i.NY == 1:
        while zoom>1:
            refd = (refd[0:runref.i.NX-1:2,:,:]+refd[1:runref.i.NX:2,:,:])/2
            zoom=zoom/2
    else:
        while zoom>1:
            refd = (refd[0:runref.i.NX-1:2,0:runref.i.NY-1:2,:]+refd[1:runref.i.NX:2,0:runref.i.NY-1:2,:]+refd[1:runref.i.NX:2,1:runref.i.NY:2,:]+refd[0:runref.i.NX-1:2,1:runref.i.NY:2,:])/4
            zoom=zoom/2
    td = (L1(d, refd)*scale, Linf(d, refd)*scale)
    print (r"%s & %s & %s \\"% (r.name,fmt,fmt)) % td

def rebin(d, f):
    nx,ny,nz = d.shape
    if ny == 1:
        assert nz == 1
        return sum(d[i:nx-f+1+i:f,:,:] for i in range(f))/f
    elif nz == 1:
        return sum(d[i:nx-f+1+i:f,i:ny-f+1+i:f,:] for i in range(f))/(f*f)
    else:
        return sum(d[i:nx-f+1+i:f,i:ny-f+1+i:f,i:nz-f+1+i:f] for i in range(f))/(f*f*f)

def table(runs, v, t=None, fmt="%12.1e", refd=None, reft=None, refr=None, scale=1, dt=0.01):
    if t is None:
        t = max(r.times[-1] for r in runs)
    print r'       Run &        $L_1$ &   $L_\infty$ \\'
    for r in runs:
        ts = None
        for n,tm in enumerate(r.times):
            if abs(tm - t) < dt:
                ts = n
                break
        if ts is None:
            print r.name + r' & crashed \\'
            continue
        d = r.var(v, ts)
        if reft is not None:
            refts = None
            for n,tm in enumerate(r.times):
                if tm == reft:
                    refts = n
                    break
            refd = r.var(v, refts)
        elif refr is not None:
            refd = refr.var(v, ts)
            if refr.i.NX != r.i.NX:
                assert (refr.i.NX//r.i.NX) * r.i.NX == refr.i.NX
                refd = rebin(refd, refr.i.NX // r.i.NX)
        td = (L1(d, refd)*scale, Linf(d, refd)*scale)
        print (r"%10s & %s & %s \\"% (r.name,fmt,fmt)) % td

def interpolate(v, x, y):
    x0 = int(numpy.floor(x))
    y0 = int(numpy.floor(y))
    x1 = int(numpy.ceil(x))
    y1 = int(numpy.ceil(y))
    if x1 >= v.shape[0]: x1 = v.shape[0]-1
    if y1 >= v.shape[1]: y1 = v.shape[1]-1
    if x0 == x1:
        if y0 == y1:
            return v[x0,y0]
        return v[x0,y0]*(y-y0) + v[x0,y1]*(y1-y)
    elif y0 == y1:
        return v[x0,y0]*(x-x0) + v[x1,y0]*(x1-x)
    return (v[x0,y0]*(x-x0) + v[x1,y0]*(x1-x))*(y-y0) + \
        (v[x0,y1]*(x-x0) + v[x1,y1]*(x1-x))*(y1-y)

def fieldlines(fx, fy, xsc=1.0, ysc=1.0, nlines=20, colour='k'):
    assert fx.shape == fy.shape
    dfy0 = sum(x for x in fy[:,0] if x > 0) / (nlines+1)
    fy0 = 0
    x0 = 0
    for li in range(nlines):
        while fy0 < dfy0:
            if fy[x0,0] > 0: fy0 += fy[x0,0]
            x0 += 1
        fy0 -= dfy0
        x = x0 - fy0/fy[x0-1,0]
        y = 0
        lx = [xsc*x]
        ly = [ysc*y]
        while x >= 0 and y >= 0 and x <= fx.shape[0] and y <= fx.shape[1]:
            dx = interpolate(fx, x, y)
            dy = interpolate(fy, x, y)
            h = numpy.sqrt(dx*dx+dy*dy)
            x += dx/h
            y += dy/h
            lx.append(xsc*x)
            ly.append(ysc*y)
        pylab.plot(lx, ly, colour)

print ""

try:
  r = Run();
  print " :: Default run loaded as \"r\"."
except:
  print " :: Default run not found!"

try:
  ref = Run('Reference');
  print " :: Reference run loaded as \"ref\"."
except:
  print " :: Reference run not found!"

try:
  mc = Run('MC');
  print " :: MC run loaded as \"mc\"."
except:
  print " :: MC run not found!"

try:
  mlmc = Run('MLMC');
  print " :: MLMC run loaded as \"mlmc\"."
except:
  print " :: MLMC run not found!"

try:
  mlmcd = Run('MLMCd');
  print " :: MLMCd run loaded as \"mlmcd\"."
except:
  print " :: MLMCd run not found!"

try:
  mc2 = Run('MC2');
  print " :: MC2 run loaded as \"mc2\"."
except:
  print " :: MC2 run not found!"

try:
  mlmc2 = Run('MLMC2');
  print " :: MLMC2 run loaded as \"mlmc2\"."
except:
  print " :: MLMC2 run not found!"

try:
  mlmc2d = Run('MLMC2d');
  print " :: MLMC2d run loaded as \"mlmc2d\"."
except:
  print " :: MLMC2d run not found!"

print ""

def tables (var):
  table([mc, mlmc, mlmcd, mc2, mlmc2, mlmc2d], var, None, "%12.1e", None, None, ref)
