
#ifndef CHAOS_H
#define CHAOS_H

#include <cstdlib>
#include <iostream>

#include "numtypes.h"
#include "WELL512a.h"
#include "primes.h"

struct Chaos {
  // real arrays only!
  real uniform [N_UNIFORM];
  real normal  [N_NORMAL];
};

static const int CHAOS_SIZE = sizeof(Chaos) / sizeof(real);

extern Chaos w;
extern Chaos *iid;
extern unsigned long int iid_size;

static real NORMAL_buffer;
static bool NORMAL_buffer_empty = true;

static void chaos_resize (unsigned long int M) {
  iid_size = M;
  iid = (Chaos*) realloc (iid, iid_size*sizeof(Chaos));
  if (iid == NULL) {
    std::cout << " :: ERROR: out of memory." << std::endl;
    throw 1;
  }
}

// generates uniform random variable on [0, 1]
static real draw_UNIFORM () {
  return WELLRNG512a();
}

// generates standard normal random variable using rejection method and Box-Muller transformation
static real draw_NORMAL () {
  
  if (!NORMAL_buffer_empty) {
    NORMAL_buffer_empty = true;
    return NORMAL_buffer;
  }
  
  else {
    real U1, U2, r, w, N1, N2;
    
    // rejection method to obtain (U1,U2) - uniform random variable on unit circle
    do {
      // draw two uniform random variable on [-1, 1]
      U1 = 2*draw_UNIFORM() - 1;
      U2 = 2*draw_UNIFORM() - 1;
      // compute radius
      r = sqr(U1) + sqr(U2);
    } while (r > 1.0);
    
    // Box-Muller transformation
    w  = sqrt(-2*log(r)/r);
    N1 = U1 * w;
    N2 = U2 * w;
    
    NORMAL_buffer = N2;
    NORMAL_buffer_empty = false;
    
    return N1;
  }
}

static Chaos generate_chaos () {
  Chaos chaos;
  
  for (int i=0; i<N_UNIFORM; i++)
    chaos.uniform[i] = draw_UNIFORM();
  
  for (int i=0; i<N_NORMAL; i++) {
    chaos.normal[i] = draw_NORMAL();
  }
  
  return chaos;
}

static void chaos_init (int offset=0) {
  // offsetting samples
  for (unsigned long int i=0; i<offset; i++)
    generate_chaos ();
  
  // generating offsetted samples
  for (unsigned long int i=0; i<iid_size; i++)
    iid[i] = generate_chaos ();
}

static void chaos_buffer_init (unsigned int seed) {
  unsigned int buffer[getR()];
  
  // init buffer using Linear Congruential Generator
  
  const unsigned long int a = 1103515245;
  const unsigned long int c = 12345;
  
  unsigned int x = seed;
  
  for (int i=0; i<getR(); i++) {
    x = a*x + c;
    buffer[i] = x;
  }
  
  InitWELLRNG512a (buffer);
}

#endif
