
/*
 * GENERAL FVM routines
 */

#include "equation.h"
#include "multi.h"

real dt, t, t_halt, halt_times[NSAVES+SAVE0], dx[NDIR];
volume<ConservedVars, 1> U;
volume<ConservedVars, 1> Flux[NDIR];
volume<ConservedVars, 1> Source;
volume<XtraVars, 1> Xtra;
int multix, multiy, multiz, multi_main;

void step_init()
{
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        calc_xtra(Xtra[i][j][k], U[i][j][k], i, j, k);
      }
    }
  }
  
  multi_bc();

  #if NX != 1
  set_ghost_x();
  #endif
  
  #if NY != 1
  set_ghost_y();
  #endif
  
  #if NZ != 1
  set_ghost_z();
  #endif
}

void model_init()
{
  PrimitiveVars pv;
  dx[X] = 1.0*MAXX/(MULTIXl*NXl);
  dx[Y] = 1.0*MAXY/(MULTIYl*NYl);
  dx[Z] = 1.0*MAXZ/(MULTIZl*NZl);
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        zero(pv);
        init_cell ( pv, (i+0.5+NXl*multix)*dx[X], (j+0.5+NYl*multiy)*dx[Y], (k+0.5+NZl*multiz)*dx[Z] );
        pv.toConservedVars(U[i][j][k]);
      }
    }
  }
}
