
/* EQUATION SPECIFIC FVM routines
 * Burgers' (scalar nonlinear conservation law)
 */

///::title:: Burgers' (scalar nonlinear conservation law)

#include "equation.h"

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{
  F.u = V.U.u * V.U.u / 2.0;
}

void apply_constraints (ConservedVars &CV, XtraVars &XV)
{
  // Constraints for ConservedVars
  /* nothing */
  // Constraints for XtraVars
  if (XV.E < 0.0) XV.E = 0.0;
}

real calc_time_step()
{
  real maxspeed = 0;

  for (int i = 0; i < NXl; i++)
    maxspeed = max(maxspeed, fabs(U[i][0][0].u));

  return CFL * dx[X] / maxspeed;
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  Xijk.E = Uijk.u * Uijk.u / 2.0;
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.E = P.u * P.u / 2.0;
}

#if INDICATOR == 1
// u | needs to be reimplemented
real indicator(int i, int j, int k, int d)
{
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.u;
}
#endif
