
#include "equation.h"
#include "hll_util.h"

void hll_speeds(const AllVariables &lV, const AllVariables &rV, int d, real &sl, real &sr, real &cs)
{
  PrimitiveVars vh;
  real wl = sqrt(lV.U.rho);
  real wr = sqrt(rV.U.rho);
  
  vh.rho = (lV.U.rho + rV.U.rho)/2;
  vh.u = (lV.XU.u * wl + rV.XU.u * wr)/(wl+wr);
  vh.p = (lV.XU.p * wl + rV.XU.p * wr)/(wl+wr);
  
  cs = sqrt(GAMMA * vh.p / vh.rho);
  
  sl = min(lV.XU.u[d] - sqrt( GAMMA * lV.XU.p / lV.U.rho ), vh.u[d] - cs);
  sr = max(rV.XU.u[d] + sqrt( GAMMA * rV.XU.p / rV.U.rho ), vh.u[d] + cs);
}

void hll_speeds_ext(const AllVariables &lV, const AllVariables &rV, int d, real eps, real &sl, real &sr, real &cs)
{
  PrimitiveVars vh;
  real wl = sqrt(lV.U.rho);
  real wr = sqrt(rV.U.rho);
  
  vh.rho = (lV.U.rho + rV.U.rho)/2;
  vh.u = (lV.XU.u * wl + rV.XU.u * wr)/(wl+wr);
  vh.p = (lV.XU.p * wl + rV.XU.p * wr)/(wl+wr);
  
  // calculate extended fast speed cf at the left boundary
  real cfl = sqrt( GAMMA*lV.XU.p/lV.U.rho );
  
  // calculate extended fast speed cf at the right boundary
  real cfr = sqrt( GAMMA*rV.XU.p/rV.U.rho );
  
  real correct = 0.5*max(0,lV.XU.u[d]-rV.XU.u[d]);
  
  cs = sqrt(GAMMA * vh.p / vh.rho);
  sl = min(lV.XU.u[d] + correct - cfl, vh.u[d] - cs);
  sr = max(rV.XU.u[d] - correct + cfr, vh.u[d] + cs);
}
