 
// legacy routines from various files

// === multi_mpi.cpp

void multi_chaos_scatter_to_levels () {
  
  resume_mpi_timer();
  
  if (L != 0) {
    int M_GLOBAL = M[0] / NSAMPLERS[0];
    
    Chaos *mpi_chaos_buf;
    if (multi_level_root) mpi_chaos_buf = new Chaos[M_GLOBAL * NSAMPLERS[0]];
    
    // gather entire iid from all samplers at level 0
    if (level_begin == 0 && multi_domain_root) {
      
      Chaos mpi_send_buf [M_GLOBAL];
      for (int i=0; i<M_GLOBAL; i++)
        mpi_send_buf[i] = iid[i];
      
      int count = M_GLOBAL * CHAOS_SIZE;
      MPI_Gather (mpi_send_buf, count, mpi_REAL, mpi_chaos_buf, count, mpi_REAL, 0, comm_subdomains);
    }
    
    // broadcast iid to all levels (level_root nodes)
    MPI_Group group_level_roots;
    MPI_Comm   comm_level_roots;
    
    MPI_Group_incl  (group_main, L+1, group_nodes[0][0][0], &group_level_roots);
    MPI_Comm_create (MPI_COMM_WORLD, group_level_roots, &comm_level_roots);
    
    if (multi_level_root) {
      int count = NSAMPLERS[0] * M_GLOBAL * CHAOS_SIZE;
      MPI_Bcast (mpi_chaos_buf, count, mpi_REAL, L, comm_level_roots);
    }
    
    // scatter samples to samplers at levels != 0
    if (level_begin != 0 && multi_domain_root) {
      
      int M_LOCAL = iid.size();
      
      Chaos mpi_recv_buf [M_LOCAL];
      
      int count = M_LOCAL * CHAOS_SIZE;
      MPI_Scatter (mpi_chaos_buf, count, mpi_REAL, mpi_recv_buf, count, mpi_REAL, 0, comm_subdomains);
      
      for (int i=0; i<M_LOCAL; i++)
        iid[i] = mpi_recv_buf[i];
    }
    
    if (multi_level_root) delete [] mpi_chaos_buf;
  }
  
  halt_mpi_timer();
}

void multi_chaos_send_to_previous_level () {
  
  resume_mpi_timer();
  
  int l = level_begin;
  
  int M_GLOBAL = M[l] / NSAMPLERS[l];
  
  if (multi_level_root) mpi_chaos_hold_buf = new Chaos[M_GLOBAL * NSAMPLERS[l]];
  
  // gather entire iid from all samplers at level 0
  if (multi_domain_root) {
    
    Chaos mpi_send_buf [M_GLOBAL];
    for (int i=0; i<M_GLOBAL; i++)
      mpi_send_buf[i] = iid[i];
    
    int count = M_GLOBAL * CHAOS_SIZE;
    MPI_Gather (mpi_send_buf, count, mpi_REAL, mpi_chaos_hold_buf, count, mpi_REAL, 0, comm_subdomains);
  }
  
  // send iid to previous level (level_root node)
  if (multi_level_root) {
    
    int count = NSAMPLERS[l] * M_GLOBAL * CHAOS_SIZE;
    MPI_Isend (mpi_chaos_hold_buf, count, mpi_REAL, group_nodes[0][0][0][L-(l-1)], 0, MPI_COMM_WORLD, &req_hold);
  }
  
  halt_mpi_timer();
}

void multi_chaos_get_from_next_level () {
  
  resume_mpi_timer();
  
  int l = level_begin;
  
  int M_GLOBAL = M[l+1] / NSAMPLERS[l+1];
  
  Chaos *mpi_chaos_recv_buf;
  
  // get iid from next level (level_root node)
  if (multi_level_root) {
    
    mpi_chaos_recv_buf = new Chaos [M_GLOBAL * NSAMPLERS[l+1]];
    
    MPI_Status s;
    int count = NSAMPLERS[l+1] * M_GLOBAL * CHAOS_SIZE;
    MPI_Recv (mpi_chaos_recv_buf, count, mpi_REAL, group_nodes[0][0][0][L-(l+1)], 0, MPI_COMM_WORLD, &s);
  }
  
  // scatter samples to samplers
  if (multi_domain_root) {
    
    int M_LOCAL = M_GLOBAL * NSAMPLERS[l+1] / NSAMPLERS[l];
    
    Chaos mpi_recv_buf [M_LOCAL];
    
    int count = M_LOCAL * CHAOS_SIZE;
    MPI_Scatter (mpi_chaos_recv_buf, count, mpi_REAL, mpi_recv_buf, count, mpi_REAL, 0, comm_subdomains);
    
    for (int i=0; i<M_LOCAL; i++)
      iid[i] = mpi_recv_buf[i];
  }
  
  if (multi_level_root) delete [] mpi_chaos_recv_buf;
  
  halt_mpi_timer();
}

void multi_cleanup () {
  // from multi_chaos_send_to_previous_level()
  if (MULTIL && !DOWNSAMPLE && level_begin != 0 && multi_level_root) {
    MPI_Wait(&req_hold, MPI_STATUS_IGNORE);
    delete [] mpi_chaos_hold_buf;
  }
}
