
#include "equation.h"
#include "chaos.h"

///::title:: Cloud Shock for Euler equations of gas dynamics
///::vars:: NX=100 NY=100
///::consts:: MAXX=1 MAXY=1 MAXT=0.06
///::consts:: N_UNIFORM=8 N_NORMAL=0
///::bc:: NEUMANN

void init_cell(PrimitiveVars &v, real x, real y, real z)
{
  if (x < 0.04 + 0.02*w.uniform[0]) {
    v.rho = 3.86859 + 0.1*w.uniform[6];
    v.u[X] = 11.2536;
    v.p = 167.345 + 1*w.uniform[7];
  } else {
    real r = sqr(x-0.25) + sqr(y-0.5);
    real phi = (x-0.25) / sqrt(r);
    if ( sqr(x-0.25) + sqr(y-0.5) <= sqr(0.13 + 0.02*w.uniform[4]*sin(phi) + 0.01*w.uniform[5]*sin(10*phi) ) )
      v.rho = 10.0 + 0.5*w.uniform[1] + 1.0*w.uniform[2]*sin(4*(x-0.25)) + 0.5*w.uniform[3]*cos(8*(y-0.5));
    else
      v.rho = 1.0;
    v.p = 1.0;
  }
}
