
#include "multi_mpi_helper.h"
#include "multi.h"
#include "main.h"

#include <iostream>
#include <time.h>
#include <fstream>
#include <cstdlib>
#include "mpi.h"

double mpi_wall_timer;
double mpi_wall_runtime = 0;
double wall_timer;

double clock_timer;
double clock_runtime;

void start_timer () {
  MPI_Barrier (MPI_COMM_WORLD);
  if (multi_main) wall_timer = MPI_Wtime();
  clock_timer = clock();
}

void stop_timer () {
  
  MPI_Barrier (MPI_COMM_WORLD);
  
  double mpi_wall_runtime_total;
  MPI_Reduce (&mpi_wall_runtime, &mpi_wall_runtime_total, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  
  clock_runtime = (clock() - clock_timer) / CLOCKS_PER_SEC;
  double clock_runtime_total;
  MPI_Reduce (&clock_runtime, &clock_runtime_total, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
  
  if (multi_main) {
    
    double wall_runtime;
    wall_runtime = MPI_Wtime() - wall_timer;
    
    double wall_runtime_total = wall_runtime * MULTI_TOTAL;
    double mpi_scaling   = 1 - mpi_wall_runtime_total/wall_runtime_total;
    if (mpi_scaling < 0) mpi_scaling = 0;
    
    std::ofstream runtimef (RUNTIME_FILE);
    runtimef << "clock_runtime=" << clock_runtime_total << std::endl;
    runtimef << "wall_runtime=" << wall_runtime << std::endl;
    runtimef << "wall_runtime_total=" << wall_runtime_total << std::endl;
    runtimef << "mpi_wall_runtime_total=" << mpi_wall_runtime_total << std::endl;
    runtimef << "mpi_scaling=" << mpi_scaling << std::endl;
    runtimef << "nodes=" << MULTI_TOTAL << std::endl;
    runtimef.flush();
    
    std::cout << " :: Clock time      (sec): " << clock_runtime_total << std::endl;
    std::cout << " :: Wall time       (sec): " << wall_runtime << std::endl;
    std::cout << " :: Wall time total (sec): " << wall_runtime_total << std::endl;
    std::cout << " :: MPI overhead    (sec): " << mpi_wall_runtime_total << std::endl;
    std::cout << " :: MPI scaling     (sec): " << mpi_scaling << std::endl;
  }
}

void resume_mpi_timer () {
  clock_runtime = (clock() - clock_timer) / CLOCKS_PER_SEC;
  mpi_wall_timer = MPI_Wtime();
}

void halt_mpi_timer () {
  mpi_wall_runtime += MPI_Wtime() - mpi_wall_timer;
  clock_timer = clock();
}

void multi_set_first_and_last_level () {
  int level = L;
  int nodes = NSAMPLERS[level] * MULTI[X][level] * MULTI[Y][level] * MULTI[Z][level];
  
  while (multi_rank >= nodes) {
    level--;
    nodes += NSAMPLERS[level] * MULTI[X][level] * MULTI[Y][level] * MULTI[Z][level];
  }
  
  // if there is no parallelisation after this level, i.e. this level includes all remaining lower levels
  if (level == LOWEST_MPI_LEVEL) {
    level_begin = 0;
    level_end   = level;
  }
  
  // else this level is paralellised as usual
  else {
    level_begin = level;
    level_end   = level;
  }
}

int get_nodes_down_to_level (int l) {
  int nodes = 0;
  int level = L;
  
  while (level > l) {
    nodes += NSAMPLERS[level] * MULTI[X][level] * MULTI[Y][level] * MULTI[Z][level];
    level--;
  }
  
  return nodes;
}

void get_group_nodes (const int multix, const int multiy, const int multiz, int *group_nodes) {
  
  int needed_multix, needed_multiy, needed_multiz;
  
  for (int l=L; l>=0; l--) {
    needed_multix = multix / (MULTIX / MULTI[X][l]);
    needed_multiy = multiy / (MULTIY / MULTI[Y][l]);
    needed_multiz = multiz / (MULTIZ / MULTI[Z][l]);
    
    group_nodes[L-l] =  get_nodes_down_to_level(l);
    group_nodes[L-l] += needed_multiz*MULTI[X][l]*MULTI[Y][l];
    group_nodes[L-l] += needed_multiy*MULTI[X][l];
    group_nodes[L-l] += needed_multix;
  }
}
