
#include "main.h"
#include "solver.h"
#include "ec_util.h"

///::title:: ES solver (Energy Stable) | for Shallow Water
///::depends:: ec_util-sw

void solver_calc_flux(ConservedVars &F, vector &us, vector &Bs,
                      const AllVariables &lV, const AllVariables &rV,
                      int i, int j, int k, int d)
{
  // === EC flux
  
  avg_flux (F, lV, rV, d);
  
  // === numerical diffusion
  
  // construction of matrix R
  
  real R [3][3] = { {1, 0, 1}, {0, 0, 0}, {0, 0, 0} };
  
  real sqrt_gh = sqrt ( GRAVITY * avg(lV.U.h, rV.U.h) );
  
  // 1st column of R
  R[1+d][0] = avg(lV.XU.u[d], rV.XU.u[d]) - sqrt_gh;
  R[2-d][0] = avg(lV.XU.u[1-d], rV.XU.u[1-d]);
  
  // 2nd column of R
  R[2-d][1] = sqrt_gh;
  
  // 3rd column of R
  R[1+d][2] = avg(lV.XU.u[d], rV.XU.u[d]) + sqrt_gh;
  R[2-d][2] = avg(lV.XU.u[1-d], rV.XU.u[1-d]);
  
  real scaling = sqrt(2*GRAVITY);
  for (int a=0; a<3; a++)
    for (int b=0; b<3; b++)
      R[a][b] /= scaling;
  
  // construction of diagonal D
  
  real D [3];
  
  D[0] = fabs ( avg(lV.XU.u[d], rV.XU.u[d]) - sqrt_gh );
  D[1] = fabs ( avg(lV.XU.u[d], rV.XU.u[d])           );
  D[2] = fabs ( avg(lV.XU.u[d], rV.XU.u[d]) + sqrt_gh );
  
  // computing diffusion matrix RDR'
  
  real DR  [3][3] = {0.0};
  real RDR [3][3] = {0.0};
  
  // DR'
  for (int a=0; a<3; a++)
    for (int b=0; b<3; b++)
      DR[a][b] += D[a] * R[b][a];
  
  // R(DR')
  for (int a=0; a<3; a++)
    for (int b=0; b<3; b++)
      for (int c=0; c<3; c++)
        RDR[a][b] += R[a][c] * DR[c][b];
  
  // construction of vector jump in vector V
  
  real V [3];
  
  V[0] = jump ( GRAVITY * (lV.U.h + lV.XU.b) - ( sqr(lV.XU.u[0]) + sqr(lV.XU.u[1]) ) / 2.0, GRAVITY * (rV.U.h + rV.XU.b) - ( sqr(rV.XU.u[0]) + sqr(rV.XU.u[1]) ) / 2.0 );
  V[1] = jump ( lV.XU.u[0], rV.XU.u[0] );
  V[2] = jump ( lV.XU.u[1], rV.XU.u[1] );
  
  // computing diffusion term
  
  real RDRV [3] = {0.0};
  
  for (int a=0; a<3; a++)
    for (int b=0; b<3; b++)
      RDRV[a] += RDR[a][b] * V[b];
  
  // adding diffusion to EC flux
  
  F.h     -= 0.5 * RDRV[0];
  F.hu[0] -= 0.5 * RDRV[1];
  F.hu[1] -= 0.5 * RDRV[2];
}
