
/*
 * Opearator overloading for ConservedVars, PrimitiveVars and XtraVars
 */

#ifndef STRUCT_OPERATOR_OVERLOADING_H
#define STRUCT_OPERATOR_OVERLOADING_H

// structures for ConservedVars, PrimitiveVars and XtraVars
#include "include.h"

// structure for AllVariables
struct AllVariables {
  ConservedVars  U;
  XtraVars      XU;
  
  AllVariables() {}
  AllVariables(ConservedVars &u, XtraVars &x): U(u), XU(x) {}
};

// structure sizes: needed for operator overloading below
static const int SIZE_CV = sizeof(ConservedVars) / sizeof(real);
static const int SIZE_PV = sizeof(PrimitiveVars) / sizeof(real);
static const int SIZE_XV = sizeof(XtraVars)      / sizeof(real);

template<class T>
static real* realCast (T &v) { return (real*)&v; }

// === member routines and operator overloading

// === ConservedVars

static void zero(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    realCast(CV) [m] = 0.0;
}

static void NonNegative(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    if (realCast(CV) [m] < 0.0) realCast(CV) [m] = 0.0;
}

static void Exp(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    realCast(CV) [m] = exp( realCast(CV) [m] );
}

static void Log(ConservedVars &CV) {
  for (int m=0; m<SIZE_CV; m++)
    realCast(CV) [m] = log( realCast(CV) [m] );
}

// needed for ALSVID

/*
static inline ConservedVars operator + (const ConservedVars &a, const ConservedVars &b);
static inline ConservedVars operator - (const ConservedVars &a, const ConservedVars &b);
static inline ConservedVars operator * (real r, const ConservedVars &a);
static inline ConservedVars operator * (const ConservedVars &a, real r);
*/

static inline ConservedVars operator + (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] + realCast(b) [m];
  return v;
}

static inline ConservedVars operator - (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] - realCast(b) [m];
  return v;
}

static inline ConservedVars operator * (real r, const ConservedVars &a) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = r * realCast(a) [m];
  return v;
}

static inline ConservedVars operator * (const ConservedVars &a, real r) { return r*a; }

// needed for MLMC

/*
static inline ConservedVars operator * (const ConservedVars &a, const ConservedVars &b);
static inline ConservedVars operator / (const ConservedVars &a, real r);
*/

static inline ConservedVars operator * (const ConservedVars &a, const ConservedVars &b) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] * realCast(b) [m];
  return v;
}

static inline ConservedVars operator / (const ConservedVars &a, real r) {
  ConservedVars v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v) [m] = realCast(a) [m] / r;
  return v;
}

// === PrimitiveVars

static void zero(PrimitiveVars &PV) {
  for (int m=0; m<SIZE_PV; m++)
    realCast(PV) [m] = 0.0;
}

/*
static inline PrimitiveVars operator + (const PrimitiveVars &a, const PrimitiveVars &b);
static inline PrimitiveVars operator - (const PrimitiveVars &a, const PrimitiveVars &b);
static inline PrimitiveVars operator * (real r, const PrimitiveVars &a);
static inline PrimitiveVars operator * (const PrimitiveVars &a, real r);
static inline PrimitiveVars operator * (const PrimitiveVars &a, const PrimitiveVars &b);
*/

static inline PrimitiveVars operator + (const PrimitiveVars &a, const PrimitiveVars &b)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] + realCast(b) [m];
  return v;
}

static inline PrimitiveVars operator - (const PrimitiveVars &a, const PrimitiveVars &b)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = realCast(a) [m] - realCast(b) [m];
  return v;
}

static inline PrimitiveVars operator * (real r, const PrimitiveVars &a)
{
  PrimitiveVars v;
  for (int m=0; m<SIZE_PV; m++)
    realCast(v) [m] = r * realCast(a) [m];
  return v;
}

static inline PrimitiveVars operator * (const PrimitiveVars &a, real r) { return r*a; }

// === XtraVars

static void zero(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    realCast(XV) [m] = 0.0;
}

static void NonNegative(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    if (realCast(XV) [m] < 0.0) realCast(XV) [m] = 0.0;
}

static void Exp(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    realCast(XV) [m] = exp( realCast(XV) [m] );
}

static void Log(XtraVars &XV) {
  for (int m=0; m<SIZE_XV; m++)
    realCast(XV) [m] = log( realCast(XV) [m] );
}

// needed for MLMC

/*
static inline XtraVars operator * (const XtraVars &a, const XtraVars &b);
static inline XtraVars operator / (const XtraVars &a, real r);
*/

static inline XtraVars operator * (const XtraVars &a, const XtraVars &b) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] * realCast(b) [m];
  return v;
}

static inline XtraVars operator / (const XtraVars &a, real r) {
  XtraVars v;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v) [m] = realCast(a) [m] / r;
  return v;
}

// === AllVariables

// needed for Shallow Water equations

static inline AllVariables operator + (const AllVariables &a, const AllVariables &b) {
  AllVariables v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v.U) [m] = realCast(a.U) [m] + realCast(b.U) [m];
  for (int m=0; m<SIZE_XV; m++)
    realCast(v.XU) [m] = realCast(a.XU) [m] + realCast(b.XU) [m];
  return v;
}

static inline AllVariables operator / (const AllVariables &a, const real r) {
  AllVariables v;
  for (int m=0; m<SIZE_CV; m++)
    realCast(v.U) [m] = realCast(a.U) [m] / r;
  for (int m=0; m<SIZE_XV; m++)
    realCast(v.XU) [m] = realCast(a.XU) [m] / r;
  return v;
}

#endif
