
/* Euler (Euler equations of gas dynamics)
 * Structures and operator overloading for ConservedVars, PrimitiveVars and XtraVars
 */

#ifndef STRUCTURES_EULER_H
#define STRUCTURES_EULER_H

#include "numtypes.h"

// === ConservedVars

struct ConservedVars {
  real   rho;
  vector m;
  real   E;
  
  real norm() const
  {
    return fabs(rho)+m.norm()+fabs(E);
  }
  
  // defined in "structures.cpp"
  
  void operator += (const ConservedVars &a);
  void operator -= (const ConservedVars &a);
  void operator *= (const ConservedVars &a);
  void operator /= (const ConservedVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === XtraVars

struct XtraVars {
  real   p;
  vector u;
  
  // defined in "structures.cpp"
  
  void operator += (const XtraVars &a);
  void operator -= (const XtraVars &a);
  void operator *= (const XtraVars &a);
  void operator /= (const XtraVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === PrimitiveVars

struct PrimitiveVars {
  real   rho;
  real   p;
  vector u;
  
  void toConservedVars(ConservedVars &v) const {
    v.rho = rho;
    v.m = rho*u;
    v.E = p/(GAMMA-1) + 0.5*rho*u.sqr();
  }
  
  PrimitiveVars() {}
  PrimitiveVars(const ConservedVars &CV, const XtraVars &XV);
};

// picks PrimitiveVars from ConservedVars (U) and XtraVars (Xtra)
inline PrimitiveVars::PrimitiveVars(const ConservedVars &CV, const XtraVars &XV)
{
  rho = CV.rho;
  p   = XV.p;
  u   = XV.u;
}

#endif
